
import weakref

class Ai:
    def __init__(self, host):
        self.host = host
        self.mode = {  }
        self.avoid = []

    def setup(self):
        pass

    def find_targets(self):
        def calc_cost(orig, dest):
            dx, dy = (orig[0]-dest[0], orig[1]-dest[1])
            return dx*dx + dy*dy


        p = self.host.coord #host map coordinates
        c = [] #target candidates
        search = self.mode[self.host.scary]["search"]

        #look for stuff adjacent to host
        px, py = p
        check = [(px-1,py), (px+1,py), (px,py+1), (px,py-1)]

        vx, vy = self.host.vel
        if vy > 0:
            check_order = [2,0,1,3]
        elif vy < 0:
            check_order = [3,0,1,2]
        elif vx > 0:
            check_order = [1,2,3,0]
        else:
            check_order = [0,2,3,1]

        right_next = False
        for ca in check_order:
            t = check[ca]
            if t in search[1] and t not in self.avoid:
                c.append(t)
                right_next = True
            elif t in search[0] and t not in self.avoid:
                c = [t]
                right_next = True
                break



        #look for all stuff on map
        if not c:
            for lst in search:
                target = None
                cost = None
                for t in lst:
                    tmp_cost = calc_cost(p, t)
                    if (target is None or cost > tmp_cost):
                       target = t
                       cost = tmp_cost
                if target:
                    c.append(target)

        sorted_c = []
        while c:
            #nearest target first
            min = None
            for t in c:
                if min is None or calc_cost(p, min) > calc_cost(t, min):
                    min = t
            sorted_c.append(min)
            c.remove(min)
        return sorted_c, right_next


    def get_vel(self):
        #return vel to next target
        class Node:
            def __init__(self, coord, g_score, prev):
                self.coord = coord
                self.g_score = g_score
                self.prev = prev

        def calc_cost(orig, dest):
            dx, dy = (orig[0]-dest[0], orig[1]-dest[1])
            return dx*dx + dy*dy

        def in_nodelist(coord, nodelist):
            for n in nodelist:
                if n.coord == coord:
                    return n
            return False


        #check if there is a cat right next to dog, that should be avoided
        avoid = []
        for cat in self.mode[self.host.scary]["avoid"]:
            if cat:
                avoid.append(cat.coord)
        self.avoid = avoid

        vx, vy = self.host.vel
        if vx > 0:
            adj_check = [1,2,3,0]
        elif vx < 0:
            adj_check = [0,2,3,1]
        elif vy > 0:
            adj_check = [3,0,1,2]
        else:
            adj_check = [2,0,1,3]


        destiny, right_next = self.find_targets()
        if not destiny:
            return None

        next_coord = None
        done = 0
        home = self.host.coord

        if right_next:
            next_coord = destiny[0]
        else:
            #print "seek your destiny...",
            for dest in destiny:
                open = [Node(home, 0, None)]
                closed = []
                while not done:
                    #search open list for node with lowest g+h score
                    #
                    #g_score = home2node distance
                    #h_score = node2dest distance (aprox.)
                    min_node = None
                    for cand in open:
                        cand_cost = cand.g_score + calc_cost(cand.coord, dest)
                        if cand.coord not in closed and \
                            (min_node is None or cand_cost < min_node.g_score):
                            min_node = cand

                    if min_node is None:
                        #print "There is no way!"
                        return None

                    #add coords adjacent to min_cost into open list
                    mx, my = min_node.coord
                    adj = [(mx-1,my), (mx+1,my), (mx,my-1), (mx,my+1)]
                    adj_cost = min_node.g_score + 1

                    for ac in adj_check:
                        a = adj[ac]
                        if a in self.ground and a not in closed and a not in avoid:
                            in_open = in_nodelist(a, open)
                            if in_open and in_open.g_score > adj_cost:
                                #if path to adj_node shorter via min_node
                                in_open.g_score = adj_cost
                                in_open.prev = min_node
                            elif not in_open:
                                #just add the new node
                                open.append(Node(a, adj_cost, min_node))
                            if a == dest:
                                done = 1 #destiny found!

                    if min_node.coord not in closed:
                        closed.append(min_node.coord)
                if done:
                    break
            #print "destiny found."
            dest_node = in_nodelist(dest, open)


            if dest_node:
                #get next step on path
                next_coord = dest_node.coord
                while dest_node.coord != home:
                    next_coord = dest_node.coord
                    dest_node = dest_node.prev

        if next_coord:
            #calc vel
            dx, dy = next_coord
            if home[0] < dx:
                vel = (1, 0)
            elif home[0] > dx:
                vel = (-1, 0)
            elif home[1] < dy:
                vel = (0, 1)
            else:
                vel = (0, -1)
            return vel
        return None

    def get_dir(self):
        #return direction to next target
        vel = self.get_vel()
        if not vel:
            return None

        if vel[1] > 0: #down
            return 0
        elif vel[1] < 0: #up
            return 1
        elif vel[0] < 0: #left
            return 2
        return 3 #right


class PacAi(Ai):
    def setup(self, map, cats):
        catnip = map.pellets
        cataway = map.cataway
        self.map_size = map.csize
        self.ground = []
        for k in map.mapdata.keys():
            m = map.mapdata[k]
            if m==0 or m==2:
                self.ground.append(k)

        #normal mode (scared)
        self.mode[False] = {
            "search": [cataway, catnip],
            "avoid": cats,
        }

        #cataway-mode (scary)
        self.mode[True] = {
            "search": [catnip, cataway],
            "avoid": [],
        }


class CatAi(Ai):
    def setup(self, map, dogs):
        self.map_size = map.csize
        self.ground = []
        for k in map.mapdata.keys():
            m = map.mapdata[k]
            if m==0 or m==2:
                self.ground.append(k)

        self.dogs = dogs
        #normal mode (scary)
        self.mode[True] = {
            "search": [[],[]],
            "avoid": [],
        }

        #cataway-mode (scared)
        self.mode[False] = {
            "search": [[], []],
            "avoid": dogs,
        }


    def get_vel(self):
        if self.host.scary:
            dcoords = []
            for d in self.dogs:
                dcoords.append(d.coord)
            self.mode[True]["search"] = [dcoords,[]]
        return Ai.get_vel(self)
