import pygame
from pygame.locals import KEYDOWN, K_ESCAPE


from widgets import SuperFlipButton
from basestates import BgState
from editor import Editor, LoadMenu
from game import StartCampaign, Campaign, Highscore
from res import *



class StartMenu(BgState):

    def __init__(self):

        def add_btn(pos, name, callback, **kwargs):
            img1 = GFX[name + "1"]
            img2 = GFX[name + "2"]
            img3 = GFX[name + "3"]
            anim = [img1, img2, img1, img3]
            btn = SuperFlipButton(img1, anim, callback, **kwargs)
            btn.set_pos(pos)
            self.add_button(name, btn)


        img = GFX["main"]
        BgState.__init__(self, img)
        pygame.mouse.set_visible(1)

        #add buttons
        pos = (29,132)
        add_btn(pos, "btn_start", self.start, state=StartCampaign, bgimg=self.background)
        pos = (29,286)
        add_btn(pos, "btn_resume", self.resume)
        pos = (29,437)
        add_btn(pos, "btn_freeplay", self.start, state=LoadMenu, edit=False)
        pos = (315,78)
        add_btn(pos, "btn_highscore", self.start, state=Highscore, bgimg=self.background)
        pos = (315,519)
        add_btn(pos, "btn_exit", self.quit)
        pos = (599,286)
        add_btn(pos, "btn_editor", self.start, state=Editor)
        pos = (599,437)
        add_btn(pos, "btn_help", self.start, state=Help)
        pos = (537,113)
        add_btn(pos, "btn_credits", self.start, state=Credits)

        self.eyes = {
            "btn_highscore": GFX["eyes_u"],
            "btn_credits": GFX["eyes_ur"],
            "btn_editor": GFX["eyes_r"],
            "btn_help": GFX["eyes_dr"],
            "btn_exit": GFX["eyes_d"],
            "btn_freeplay": GFX["eyes_dl"],
            "btn_resume": GFX["eyes_l"],
            "btn_start": GFX["eyes_ul"],
        }
        self.redraw()

        self.add_kbevent(KEYDOWN, K_ESCAPE, self.quit)

    def redraw(self):
        for b in self.buttons.keys():
            self.buttons[b].draw(self.background)

    def update_screen(self):
        self.background.blit(GFX["main"], (0,0))
        for b in self.buttons.keys():
            btn = self.buttons[b]
            if btn.hover:
                btn.animate()
                eye = self.eyes[b]
                self.background.blit(eye, (293,235))
                self.background.blit(eye, (425,235))

            btn.draw(self.background)

        self.screen.blit(self.background, (0,0))
        pygame.display.flip()

    def main_start(self):
        BgState.main_start(self)
        pygame.mouse.set_visible(1)
        self.redraw()

    def start(self, state, **kwargs):
        self.pause()
        self.next = state(**kwargs)
        self.quit()

    def resume(self):
        if SAVE["cur"]:
            self.start(Campaign)
        #else:
        #    self.start(StartCampaign, bgimg=self.background)

    def unpause(self):
        BgState.unpause(self)
        x,y = pygame.mouse.get_pos()
        pygame.mouse.set_pos((x+1, y))


class Help(BgState):
    def __init__(self):
        img = GFX["help"]
        BgState.__init__(self, img)

        img1 = GFX["btn_back1"]
        img2 = GFX["btn_back2"]
        img3 = GFX["btn_back3"]
        anim = [img1, img2, img1, img3]
        btn = SuperFlipButton(img1, anim, callback=self.quit)
        btn.set_pos((315,519))
        self.add_button("back", btn)


    def update_screen(self):
        btn = self.buttons["back"]
        if btn.hover:
            btn.animate()
            btn.draw(self.background)
        self.screen.blit(self.background, (0,0))
        pygame.display.flip()


class Credits(Help):
    def __init__(self):
        Help.__init__(self)
        self.background.blit(GFX["credits"], (0,0))




